/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "find.h"

void search_all_files (vector<Glib::ustring>&, Glib::ustring, Glib::ustring, 
                       Glib::ustring, Glib::ustring);

// This function creates the find toolbar by initializing the widgets, creating
// the buttons and adding them to the horizontal box. The buttons are then
// connected to their signals
void OpenLDev::populate_find_toolbar()
{
  // Create Horizontal Find 'Toolbar'
  hbox_find = new Gtk::HBox(false, 0);
  label_find = new Gtk::Label("Search: ");
  label_error = new Gtk::Label("Text not found!");
  find_text = new Gtk::Entry();
  image_close = new Gtk::Image(Gtk::StockID("gtk-stop"), Gtk::IconSize(1));
  button_find = new Gtk::Button(Gtk::StockID("gtk-find"));
  button_replace = new Gtk::Button(Gtk::StockID("gtk-find-and-replace"));
  button_close = new Gtk::Button();
  image_error = new Gtk::Image(Gtk::StockID("gtk-dialog-error"), Gtk::IconSize(1));
  search_files = new Gtk::CheckButton("Search in all files");
  
  // Create the buttons
	button_close->set_relief(Gtk::RELIEF_NONE);
	button_close->add(*image_close);
  button_find->set_relief(Gtk::RELIEF_NONE);
  button_replace->set_relief(Gtk::RELIEF_NONE);
  search_files->set_active(false);
  
  // Add the widgets to the horizontal box
  hbox_find->set_spacing(5);
  hbox_find->set_border_width(3);
  hbox_find->pack_start(*button_close, Gtk::PACK_SHRINK);
  hbox_find->pack_start(*label_find, Gtk::PACK_SHRINK);
  hbox_find->pack_start(*find_text, Gtk::PACK_SHRINK);
  hbox_find->pack_start(*button_find, Gtk::PACK_SHRINK, 5);
  hbox_find->pack_start(*button_replace, Gtk::PACK_SHRINK);
  hbox_find->pack_start(*search_files, Gtk::PACK_SHRINK, 5);
  hbox_find->pack_start(*image_error, Gtk::PACK_SHRINK);
  hbox_find->pack_start(*label_error, Gtk::PACK_SHRINK);
  
  // Connect the buttons to their signals
  button_close->signal_clicked().connect(sigc::mem_fun(*this, &OpenLDev::close_toolbars));
  button_find->signal_clicked().connect(sigc::mem_fun(*this, &OpenLDev::on_find_next));
  button_replace->signal_clicked().connect(sigc::mem_fun(*this, &OpenLDev::find_and_replace));
}

// Toggle the visibility of the find toolbar. Also, this makes sure that if
// the Find toolbar is visible, the goto line toolbar is hidden
void OpenLDev::on_edit_find() 
{
  if (hbox_find->is_visible())
    on_find_next();
  else
  {
    hbox_find->show();
    hbox_goto->hide();
  }
}

// Find the next iteration of the text the user specified in the buffer.
// If there is no selected text, search from the current cursor position.
// Otherwise, search from the selection bound. If the text is not found, wrap
// around the document once and then throw an error if it is still not found.
void OpenLDev::on_find_next()
{
  // Check if there was no text specified
  if (!find_text->get_text().empty())
  {
    if (search_files->get_active())
    {
      notebook->set_current_page(1);
      search_box->remove_all_columns();
      vector<Glib::ustring> search_results;
      search_all_files(search_results, proj.get_source_files(), proj.get_header_files(), 
                       proj.get_location(), find_text->get_text());
      
      if (search_results.size() > 0)
      {
        BuildColumns columns;
        Glib::RefPtr<Gtk::ListStore> model;
        Gtk::CellRendererText *renderer;
        Gtk::TreeModel::Row row;
        Gtk::TreeViewColumn *color_column;
  
        model = Gtk::ListStore::create(columns);
        search_box->set_model(model);
  
        search_box->append_column("Search Output", columns.col_text);
        color_column = search_box->get_column(0);
        Gtk::CellRenderer *cell_renderer = color_column->get_first_cell_renderer();
        renderer = dynamic_cast<Gtk::CellRendererText*>(cell_renderer);
        color_column->add_attribute(renderer->property_foreground(), columns.col_color);
  
        for (unsigned int i=0; i < search_results.size(); i++)
        {
          row = *(model->append());
          row[columns.col_text] = search_results.at(i);
          row[columns.col_color] = "#990000";
        }
        
        label_error->hide();
        image_error->hide();
      }
      else
      {
        label_error->show();
        image_error->show();
      }
    }
    else
    {
      Glib::RefPtr<Gtk::TextBuffer> buffer = noteman->get_textbuffer();
      Gtk::TextIter iter, end, enditer = buffer->end();
  
      // If there is no selected text, then search from the cursor position
      if (buffer->get_selection_bound()->get_iter() > buffer->get_insert()->get_iter())
        iter = buffer->get_selection_bound()->get_iter();
      else
        iter = buffer->get_insert()->get_iter();
  
      // If it is found, select the text
      if (iter.forward_search(find_text->get_text(), Gtk::TEXT_SEARCH_VISIBLE_ONLY,iter,end,enditer)) 
        mark_found_text(iter, end);
      // Throw an error if it cannot be found
      else 
      {
        iter = buffer->begin();
        if (iter.forward_search(find_text->get_text(), Gtk::TEXT_SEARCH_VISIBLE_ONLY,iter,end,enditer)) 
          mark_found_text(iter, end);
        else
        {
          label_error->show();
          image_error->show();
        }
      }
    }
  }
}

// Mark the found text by selecting it. This function receives the beginning
// and end iterators of the found text. When this function is called, the error
// message and image on the find toolbar is hidden
void OpenLDev::mark_found_text(Gtk::TextIter iter, Gtk::TextIter end)
{
  Glib::RefPtr<Gtk::TextBuffer> buffer = noteman->get_textbuffer();
  buffer->move_mark(buffer->get_insert(), iter);
  buffer->move_mark(buffer->get_selection_bound(), end);
  noteman->scroll_to_mark();
  set_focus(*noteman->get_textview());
      
  label_error->hide();
  image_error->hide();
}

// Search all of the files in a project for the text you want to find. This
// function receives the vector of search results to edit by reference, and
// strings with the source and header files, the location of the project
// directory and the text to search for in the files.
void search_all_files (vector<Glib::ustring> &search_results, Glib::ustring sources,
                       Glib::ustring headers, Glib::ustring location,
                       Glib::ustring find_text)
{
  Glib::ustring files = sources + headers;
  
  // while there are files left to search
  while (files.length() > 0)
  {
    // Create the file string & erase it from the list
    int loc = files.find('/', 0), line = 1;
    string text = location + "src/" + files.substr(0, loc), file = files.substr(0, loc);
    files.erase(0, loc+1);
    
    // Open the file & get it line by line
    ifstream fin(text.c_str());
    while (getline(fin, text))
    {
      // If the text is found, add a string to the search results vector in
      // the form - filename:line: Line Content
      if (text.find(find_text, 0) != Glib::ustring::npos)
      {
        gchar buff[5];
        snprintf(buff, sizeof(buff), "%d", line);
        search_results.push_back(Glib::ustring(file + ":" + buff + ": " + text));
      }
      
      line++;
    }
    
    fin.close();
  }
}

// Catch the search box selection & do action. This function receives the button
// event. If it is a double click, get the text of the selection and then the 
// file containing the search text is opened. The cursor is then moved line of 
// the search result
void OpenLDev::search_box_selected(GdkEventButton *event)
{
  if (event->type == GDK_2BUTTON_PRESS)
  {
    Glib::RefPtr<Gtk::TreeSelection> selection = search_box->get_selection();
    Gtk::TreeModel::iterator iter = selection->get_selected();
    
    if (iter)
    {
      Gtk::TreeModel::Row row = *iter;
      Glib::ustring output, file;
      int linenumber;
      row->get_value(0, output);
        
      // Open the file the error was found in
      linenumber = get_file_name_and_line(output) - 1;
      open_file(proj.get_location() + "src/" + output);
        
      // Move the cursor to the line of the error & scroll to the cursor
      Glib::RefPtr<Gtk::TextBuffer> buffer = noteman->get_textbuffer();
      Gtk::TextIter iter = buffer->get_iter_at_line(linenumber);
      buffer->move_mark(buffer->get_insert(), iter);
      buffer->move_mark(buffer->get_selection_bound(), iter);
      noteman->scroll_to_mark();
      set_focus(*noteman->get_textview());
    }
  }
}

// Replace the specified text with the text specified by the user in the input
// dialog. By choosing the checkbox to search in all files, you can replace the
// text in all of the files.
void OpenLDev::find_and_replace()
{
  int count = 0;
  bool process_replace = true;
  Glib::ustring replace_text;
  on_find_next();
  
  // Loop until the text is no longer found
  while (noteman->get_insert_offset() != noteman->get_selection_offset())
  {
    // Get the text to replace with from the user
    if (count == 0)
    {
      InputDialog *dialog;
      dialog = new InputDialog ("Replace With: ", "Replace Text");
      int result = dialog->run();
      
      // Either say to stop replace or get the text to replace with
      switch (result)
      {
        case Gtk::RESPONSE_OK:
          replace_text = dialog->get_entry_text();
          break;
        default:
          process_replace = false;
          break;
      }
      dialog->hide();      
    }
    
    // Cancel the replacing
    if (!process_replace)
      break;
    
    // Erase the found text & replace it
    noteman->get_textbuffer()->erase(noteman->get_textbuffer()->get_insert()->get_iter(), 
             noteman->get_textbuffer()->get_selection_bound()->get_iter());
    noteman->get_textbuffer()->insert_at_cursor(replace_text);
    
    // Refresh the cursor and find the next iteration
    noteman->get_textbuffer()->move_mark_by_name("insert", 
             noteman->get_textbuffer()->get_selection_bound()->get_iter());
    on_find_next();
    count++;
    
    noteman->get_aman().erase();
  }
  
  // Hide the error if there was text found
  if (count > 0)
  {
    label_error->hide();
    image_error->hide();
  }
}
