/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef NOTEBOOKMANAGER_H
#define NOTEBOOKMANAGER_H

#include "notebooktab.h"

// Notebook manager class that handles the tabs
class NotebookManager
{
public:
	NotebookManager();
	~NotebookManager() { }

  // Functions to get private data and widgets
	Gtk::Notebook* get_notebook();
  int get_current_tab();
  Glib::ustring get_current_filename();
  Glib::ustring get_current_short_filename();
  bool get_file_open();
  Gtk::TextView* get_textview();
  Glib::RefPtr<Gtk::TextBuffer> get_textbuffer();
  ActionManager& get_aman();
  int get_insert_offset();
  int get_selection_offset();
  
  // Tab management functions
	bool add_tab(Glib::ustring, EnvironmentSettings);
	void delete_tab(Glib::ustring);
  void erase_all_tabs();
  void save_all_files();
  void mark_current_tab_modified(bool);
  void sync_scrollbars();
  void notebook_event(GdkEvent*);
  
  // Textview modification functions
  void scroll_to_mark();
  void modify_fonts(Pango::FontDescription);
  void update_line_numbers();
  void create_line_numbers();
  void show_line_numbers(bool);
  void revert_to_saved();
	
private:
	int tab_count;
	deque<NotebookTab> tabs;
	Gtk::Notebook* notebook;
};

#endif
